@@ -1,4 +1,4 @@
-from typing import Any, Dict, Type, TypeVar, List, Set, Dict, Callable
+from typing import Any, Dict, Optional, Type, TypeVar, List, Set, Dict, Callable
 from dataclasses import is_dataclass, asdict
 
 import sys
@@ -10,10 +10,15 @@ def _transformer_noop(source: str) -> str:
     return source
 
 
+def _on_extra_field_noop(*args):
+    pass
+
+
 def _to_dataclass_38(
     dataclass_type: Type[T],
     data: Any,
     key_transformer: Callable[[str], str],
+    on_extra_field: Callable[[Type, str, Any], None],
 ) -> T:
     from typing import get_args, get_type_hints, get_origin
 
@@ -25,10 +30,13 @@ def _to_dataclass_38(
     assert isinstance(data, dict)
 
     dataclass_dict: Dict[str, Any] = dict()
+    used_keys: Set[str] = set()
 
     for annotation_name, annotation_type in get_type_hints(dataclass_type).items():
         try:
-            dict_value = data[key_transformer(annotation_name)]
+            key = key_transformer(annotation_name)
+            dict_value = data[key]
+            used_keys.add(key)
         except KeyError:
             # TODO(braynstorm):
             #   Check if the field has default value. Only "continue" if it does.
@@ -39,25 +47,36 @@ def _to_dataclass_38(
         if origin is set:
             value_type = get_args(annotation_type)[0]
             dataclass_dict[annotation_name] = set(
-                _to_dataclass_38(value_type, v, key_transformer) for v in dict_value
+                _to_dataclass_38(value_type, v, key_transformer, on_extra_field)
+                for v in dict_value
             )
         elif origin is list:
             value_type = get_args(annotation_type)[0]
             dataclass_dict[annotation_name] = [
-                _to_dataclass_38(value_type, v, key_transformer) for v in dict_value
+                _to_dataclass_38(value_type, v, key_transformer, on_extra_field)
+                for v in dict_value
             ]
         elif origin is dict:
             key_type, value_type = get_args(annotation_type)
             assert key_type == str
             dataclass_dict[annotation_name] = {
-                key: _to_dataclass_38(value_type, value, key_transformer)
+                key: _to_dataclass_38(
+                    value_type, value, key_transformer, on_extra_field
+                )
                 for key, value in dict_value.items()
             }
         else:
             dataclass_dict[annotation_name] = _to_dataclass_38(
-                annotation_type, dict_value, key_transformer
+                annotation_type, dict_value, key_transformer, on_extra_field
             )
 
+    extra_keys = set(data.keys())
+    extra_keys.difference_update(used_keys)
+
+    for field in extra_keys:
+        value = data[field]
+        on_extra_field(dataclass_type, field, value)
+
     return dataclass_type(**dataclass_dict)
 
 
@@ -65,6 +84,7 @@ def _to_dataclass_37(
     dataclass_type: Type[T],
     data: Any,
     key_transformer: Callable[[str], str],
+    on_extra_field: Callable[[Type, str, Any], None],
 ) -> T:
     dict_value: Any
 
@@ -74,10 +94,20 @@ def _to_dataclass_37(
     assert isinstance(data, dict)
 
     dataclass_dict: Dict[str, Any] = dict()
+    used_keys: Set[str] = set()
 
-    for annotation_name, annotation_type in dataclass_type.__annotations__.items():
+    annotations = dict()
+    for c in dataclass_type.mro():
         try:
-            dict_value = data[key_transformer(annotation_name)]
+            annotations.update(c.__annotations__)
+        except AttributeError:
+            pass
+
+    for annotation_name, annotation_type in annotations.items():
+        try:
+            key = key_transformer(annotation_name)
+            dict_value = data[key]
+            used_keys.add(key)
         except KeyError:
             # TODO(bozho2):
             #   Check if the field has default value. Only "continue" if it does.
@@ -90,7 +120,8 @@ def _to_dataclass_37(
         ):
             value_type = annotation_type.__args__[0]
             dataclass_dict[annotation_name] = set(
-                _to_dataclass_37(value_type, v, key_transformer) for v in dict_value
+                _to_dataclass_37(value_type, v, key_transformer, on_extra_field)
+                for v in dict_value
             )
         elif (
             isinstance(annotation_type, List.__class__)
@@ -98,7 +129,8 @@ def _to_dataclass_37(
         ):
             value_type = annotation_type.__args__[0]
             dataclass_dict[annotation_name] = [
-                _to_dataclass_37(value_type, v, key_transformer) for v in dict_value
+                _to_dataclass_37(value_type, v, key_transformer, on_extra_field)
+                for v in dict_value
             ]
         elif (
             isinstance(annotation_type, Dict.__class__)
@@ -108,20 +140,29 @@ def _to_dataclass_37(
             value_type = annotation_type.__args__[1]
             assert key_type == str
             dataclass_dict[annotation_name] = {
-                k: _to_dataclass_37(value_type, v, key_transformer)
+                k: _to_dataclass_37(value_type, v, key_transformer, on_extra_field)
                 for k, v in dict_value.items()
             }
         else:
             dataclass_dict[annotation_name] = _to_dataclass_37(
-                annotation_type, dict_value, key_transformer
+                annotation_type, dict_value, key_transformer, on_extra_field
             )
+
+    extra_keys = set(data.keys())
+    extra_keys.difference_update(used_keys)
+
+    for field in extra_keys:
+        value = data[field]
+        on_extra_field(dataclass_type, field, value)
+
     return dataclass_type(**dataclass_dict)
 
 
 def to_dataclass(
     dataclass_type: Type[T],
     data: Any,
-    key_transformer: Callable[[str], str] | None = None,
+    key_transformer: Optional[Callable[[str], str]] = None,
+    on_extra_field: Optional[Callable[[Type, str, Any], None]] = None,
 ) -> T:
     """
     Convert nested dicts/lists to a dataclass structure.
@@ -158,7 +199,10 @@ def to_dataclass(
         key_transformer (Callable[[str], str] | None, optional):
             Name transformer function, to convert dataclass field names to
             names of the input data. Defaults to None - no transformation.
-
+        on_extra_field (Callable[[Type, str, Any], None] | None, optional):
+            Called when to_dataclass finds a field in a dictionary that is
+            not requested by the dataclass it is filling.
+            Args: the dataclass type, name of the field, value of the field.
     Returns:
         T: _description_
 
@@ -167,10 +211,13 @@ def to_dataclass(
     if key_transformer is None:
         key_transformer = _transformer_noop
 
+    if on_extra_field is None:
+        on_extra_field = _on_extra_field_noop
+
     if sys.version_info.minor >= 8:
-        return _to_dataclass_38(dataclass_type, data, key_transformer)
+        return _to_dataclass_38(dataclass_type, data, key_transformer, on_extra_field)
     else:
-        return _to_dataclass_37(dataclass_type, data, key_transformer)
+        return _to_dataclass_37(dataclass_type, data, key_transformer, on_extra_field)
 
 
 def dataclass_to_dict(dataclassObject: Any) -> Dict[str, Any]:
